import os
import json
from voxjar.push_request import PushRequest
import gql
from voxjar.transport import HttpTransport
import voxjar.auth


class Client(object):
    """Voxjar API client.

    Args:
        url (str, optional): The URL for the API.
        token (str, optional): The JWT authenticating this Client to the API.
        timeout (int, optional): The timeout for API calls.
    """

    def __init__(self, url=None, token=None, timeout=None):
        self.url, self.token = voxjar.auth.credentials(url=url, token=token)

        graphql_url = '{}/graphql'.format(self.url)
        self._client = gql.Client(
            transport=HttpTransport(graphql_url, token, timeout),
            fetch_schema_from_transport=True)

    def push_request(self, metadata, audio=None):
        """A file-like object used to enqueue a call audio file to be processed
        by Voxjar.

        Args:
            metadata (dict): The metadata for the call.
            audio (file): The raw audio bytes of the call.
        Returns:
            metadata (dict): The metadata for the uploaded call.
        Example:
            .. code:: python

                import datetime
                import requests
                import voxjar

                metadata = {
                    'identifier': 'callIdentifier',
                    'type': {
                        'identifier': 'typeIdentifier',
                        'name': 'typeName',
                    },
                    'timestamp': datetime.datetime.now(),
                    'direction': 'OUTGOING',  # or INCOMING
                    'agents': [{
                        'identifier': 'agentIdentifier',
                        'name': 'Agent Name',
                        'phoneNumber': 1234567890  # optional
                    }],
                    # optional
                    'customers': [{
                        'identifier': 'customerIdentifier',
                        'name': 'Customer Name',
                        'phoneNumber': 9876543210  # optional
                    }],
                    'disposition': {
                        'identifier': 'dispositionIdentifier',
                        'name': 'dispositionName',
                    },
                    'tags': ['tag1', 'tag2'],
                }

                client = voxjar.Client()

                r = requests.get('https://somesource.com', stream=True)
                with client.push_request(metadata) as push_request:
                    for chunk in r.iter_content(chunk_size=1024):
                        push_request.write(chunk)
                    push_request.push()

        """
        return PushRequest(self.token, metadata, audio=audio, url=self.url)

    def push(self, metadata, audio):
        """Enqueue a call to be processed by Voxjar.

        Args:
            metadata (dict): The metadata for the call.
            audio (file): The raw audio bytes of the call.
        Returns:
            metadata (dict): The metadata for the uploaded call.
        Example:
            .. code:: python

                import datetime
                import voxjar

                metadata = {
                    'identifier': 'callIdentifier',
                    'type': {
                        'identifier': 'typeIdentifier',
                        'name': 'typeName',
                    },
                    'timestamp': datetime.datetime.now(),
                    'direction': 'OUTGOING',  # or INCOMING
                    'agents': [{
                        'identifier': 'agentIdentifier',
                        'name': 'Agent Name',
                        'phoneNumber': 1234567890  # optional
                    }],
                    # optional
                    'customers': [{
                        'identifier': 'customerIdentifier',
                        'name': 'Customer Name',
                        'phoneNumber': 9876543210  # optional
                    }],
                    'disposition': {
                        'identifier': 'dispositionIdentifier',
                        'name': 'dispositionName',
                    },
                    'tags': ['tag1', 'tag2'],
                }

                client = voxjar.Client()

                with open('test.wav', 'rb') as f:
                    client.push(metadata, f)
        """
        with PushRequest(self.token, metadata, audio, self.url) as push_request:
            return push_request.push()

    def execute(self, document, *args, **kwargs):
        doc = gql.gql(document) if isinstance(document, str) else document
        return self._client.execute(doc, *args, **kwargs)



from io import BytesIO
import json
import datetime
import requests

try:
    from json.decoder import JSONDecodeError
except ImportError:
    JSONDecodeError = ValueError


def _serialize(obj):
    """JSON serializer for objects not serializable by default json code"""

    if isinstance(obj, (datetime.datetime, datetime.date)):
        serial = obj.isoformat()
        return serial
    raise TypeError("Type {} not serializable".format(type(obj)))


class PushRequest(object):
    """A file-like object used to enqueue a call audio file to be processed by
    Voxjar.

    Args:
        metadata (:obj:`dictionary`): The metadata for the call.
        audio (:obj:`file-like object`): The raw audio bytes of the call.
        url (str, optional): The URL for the API.
    """

    def __init__(self, token, metadata, audio=None,
                 url='https://api.voxjar.com:9000'):

        self.metadata = metadata
        self.token = token
        self.url = url
        if not audio:
            audio = BytesIO()
        self.buffer = audio

    def __enter__(self):
        return self

    def read(self, *args, **kwargs):
        return self.buffer.read(*args, **kwargs)

    def write(self, chunk):
        return self.buffer.write(chunk)

    def push(self):
        """Enqueue a call to be processed by Voxjar.

        Returns:
            metadata (:obj:`dictionary`): The metadata for the uploaded call.
        """
        self.buffer.seek(0)

        payload = json.dumps(self.metadata, default=_serialize)
        files = {
            'metadata': (None, payload, 'application/json'),
            'file': (self.metadata.get('identifier'), self.buffer,
                     'application/octet-stream')
        }
        headers = {"Authorization": "Bearer {}".format(self.token)}
        url = '{}/enqueue'.format(self.url)

        r = requests.post(url=url, files=files, headers=headers)
        try:
            return r.json()
        except JSONDecodeError:
            raise RuntimeError(r.content)

    def close(self):
        self.buffer.close()

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()



import requests
from graphql.execution import ExecutionResult
from graphql.language.printer import print_ast


class HttpTransport(object):

    def __init__(self, url, token, timeout=None, headers=None):
        """
        Args:
            url (str): The GraphQL URL
            timeout (int, optional): Specifies a default timeout for requests
                                     (Default: None)
        """
        self.url = url
        self.default_timeout = timeout
        self.token = token
        if not headers:
            self.headers = {}
        if self.token:
            self.headers['Authorization'] = 'Bearer {}'.format(self.token)

    def execute(self, document, variable_values=None, timeout=None):
        payload = {
            'query': print_ast(document),
            'variables': variable_values or {}
        }

        # TODO: check for file objects: hasattr(fp, 'read')

        post_args = {
            'headers': self.headers,
            'timeout': timeout or self.default_timeout,
            'json': payload
        }
        request = requests.post(self.url, **post_args)
        request.raise_for_status()

        result = request.json()
        assert 'errors' in result or 'data' in result,\
               'Received non-compatible response "{}"'.format(result)
        return ExecutionResult(
            errors=result.get('errors'), data=result.get('data'))



import os
import json


def credentials(url=None, token=None):
    """Get the credentials configuration from the
    `VOXJAR_APPLICATION_CREDENTIALS` environment variable.

    url can be overriden with the `VOXJAR_API_URL` environment variable.
    token can be overriden with the `VOXJAR_API_TOKEN` environment variable.

    Returns:
        dictionary: The contents of the :ref:`keyfile`.
    """
    if not url:
        url = os.getenv('VOXJAR_API_URL')

    if not token:
        token = os.getenv('VOXJAR_API_TOKEN')

    if not url or not token:
        creds = {}
        creds_file = os.getenv('VOXJAR_APPLICATION_CREDENTIALS')
        if creds_file:
            with open(creds_file, 'r') as f:
                creds = json.loads(f.read())

        if not url:
            url = creds.get('url', 'https://api.voxjar.com:9000')

        if not token:
            token = creds.get('token')

    return url, token



#!/usr/bin/env python
from __future__ import print_function
import os
import shutil
import glob
import subprocess
import setuptools

package_dir = 'voxjar'


class CustomCleanCommand(setuptools.Command):
    description = 'remove build files'
    user_options = []

    def initialize_options(self):
        """Set default values for options."""
        pass

    def finalize_options(self):
        """Post-process options."""
        pass

    def run(self):
        """Run clean."""

        def respect_dry_run(path, fn):
            if self.dry_run:
                print('would remove {}...'.format(path))
            else:
                if self.verbose > 1:
                    print('removing {}...'.format(path))
                fn(path)

        for d in [
                './dist/', './build/', './__pycache__/',
                './{}.egg-info/'.format(package_dir)
        ]:
            if os.path.isdir(d):
                respect_dry_run(d, shutil.rmtree)

        for f in glob.glob('*.pyc'):
            respect_dry_run(f, os.remove)


class CustomDocsCommand(setuptools.Command):
    description = 'build documentation'
    user_options = []

    def initialize_options(self):
        """Set default values for options."""
        pass

    def finalize_options(self):
        """Post-process options."""
        pass

    def run(self):
        """Run build docs."""
        shutil.rmtree('./docs/_build')
        subprocess.check_output(['make', '-C', 'docs', 'html'])
        subprocess.check_output(['open', './docs/_build/html/index.html'])


with open("README.rst") as readme_file:
    readme = readme_file.read()

setuptools.setup(
    name='voxjar',
    version='0.4.3',
    description='python implementation of the Voxjar API',
    long_description=readme,
    author='William Myers',
    author_email='will@voxjar.com',
    url='https://github.com/voxjar/voxjar',
    install_requires=['requests', 'gql'],
    extras_require={'docs': ['sphinx'],
                    'dev': ['wheel', 'twine']},
    python_requires='>=2.7',
    packages=setuptools.find_packages(exclude=['docs', 'tests*']),
    cmdclass={
        'clean': CustomCleanCommand,
        'docs': CustomDocsCommand,
    },
    license='Apache-2',
    keywords='voxjar voxjar ai requests',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
    ])
